#pragma semicolon 1
#include <sourcemod>
#include <sdktools>
#include <monster>
#undef REQUIRE_EXTENSIONS
#include <clientprefs>
#include <SteamWorks>

#define ACHIEVEMENT_PARTICLE "achieved"

ConVar g_hDefPoints;
Handle g_hPointCookie;

int g_iPoints[MAXPLAYERS + 1];

public Plugin myinfo =
{
	name = "Monster Core",
	description = "Manages file precaches/downloads of monster stuff and adds a little bit of logic for coop maps",
	author = "ZombieX2 and donrevan",
	version = "1.1",
};

public APLRes AskPluginLoad2(Handle myself, bool late, char[] error, int err_max)
{
	CreateNative("Monster_GetPoints", Native_GetPoints);
	CreateNative("Monster_SetPoints", Native_SetPoints);
	return APLRes_Success;
}

public void OnPluginStart()
{
	g_hDefPoints = CreateConVar("monster_defaultpoints", "5", "Default amount of points given upon map completion");
	
	RegServerCmd("sm_mapend", Command_MapEnd, "Function that is called to signal successfull completion of the map");
	RegConsoleCmd("sm_points", Command_Points, "Display current points");

	if (LibraryExists("clientprefs"))
	 {
		OnLibraryAdded("clientprefs");
	}
}

public void OnMapStart()
{
	PrecacheModel("models/headcrabclassic.mdl",true);
	PrecacheModel("models/headcrab.mdl",true);
	PrecacheModel("models/headcrabblack.mdl",true);

	PrecacheModel("models/zombie/classic.mdl",true);
	PrecacheModel("models/zombie/classic_torso.mdl",true);
	PrecacheModel("models/zombie/classic_legs.mdl",true);
	PrecacheModel("models/zombie/fast.mdl",true);
	PrecacheModel("models/zombie/Fast_torso.mdl",true);
	PrecacheModel("models/gibs/fast_zombie_torso.mdl",true);
	PrecacheModel("models/gibs/fast_zombie_legs.mdl",true);
	PrecacheModel("models/zombie/poison.mdl",true);
	
	PrecacheModel("models/manhack.mdl",true);
	
	PrecacheModel("models/antlion.mdl",true);
	PrecacheModel("models/antlion_worker.mdl",true);
	PrecacheModel("models/antlion_guard.mdl",true);
	
	PrecacheModel("models/stalker.mdl",true);
	
	PrecacheModel("models/vortigaunt.mdl",true);

	PrecacheModel("models/roller.mdl",true);
	PrecacheModel("models/roller_spikes.mdl",true);

	PrecacheModel("sprites/grubflare1.vmt",true);
	PrecacheModel("sprites/glow1.vmt",true);
	PrecacheModel("sprites/laser.vmt",true);
	PrecacheModel("sprites/redglow1.vmt",true);
	PrecacheModel("sprites/orangeglow1.vmt",true);
	PrecacheModel("sprites/yellowglow1.vmt",true);
	PrecacheModel("sprites/lgtning.vmt",true);
	PrecacheModel("sprites/vortring1.vmt",true);
	
	PrecacheModel("sprites/bluelight1.vmt",true);
	PrecacheModel("sprites/rollermine_shock.vmt",true);
	PrecacheModel("sprites/rollermine_shock_yellow.vmt",true);
	
	PrecacheModel("models/props_combine/headcrabcannister01a.mdl",true);
	PrecacheModel("models/props_combine/headcrabcannister01b.mdl",true);
	PrecacheModel("models/props_combine/headcrabcannister01a_skybox.mdl",true);

	ReadDownloadFile();
	 
	 PrintToServer("[MONSTER] monster_core has been loaded successfully!");
}

public void OnClientCookiesCached(int client)
{
	 char value[8];
	 GetClientCookie(client, g_hPointCookie, value, sizeof(value));
	 
	 g_iPoints[client] = (value[0] != '\0' && StringToInt(value));
}

public SteamWorks_SteamServersConnected()
{
	 // yup
	 SteamWorks_SetGameDescription("Cooperative");
}

static bool IsCommentLine(const char[] line, int maxlen)
{
	 int pos = 0;
	 do {
		  if(line[pos] == '#')
				return true;
	 } while(line[pos++] == ' ' && pos < maxlen);
	 return false;
}

void ReadDownloadFile()
{
	char path[PLATFORM_MAX_PATH];
	BuildPath(Path_SM, path, sizeof(path), "configs/monster/downloads.txt");
	
	File file = OpenFile(path, "rt");
	if (file == INVALID_HANDLE)
		return;
	
	char readData[256];
	while (!IsEndOfFile(file) && ReadFileLine(file, readData, sizeof(readData)))
	{
		TrimString(readData);
		int length = strlen(readData);
		if (length >= 4)
		{
				if (IsCommentLine(readData, sizeof(readData)))
				{
					 continue;
				}
		  
				if (!FileExists(readData))
				{
					 PrintToServer("[MONSTER] %s Not Exist!",readData);
					 continue;
				}
		  
			if (strcmp(readData[length-4],".mdl") == 0)
			{
				PrecacheModel(readData,true);
			}
				
				AddFileToDownloadsTable(readData);
		}
	}
	CloseHandle(file);
}

public void OnLibraryAdded(const char[] name)
{
	if(strcmp(name, "clientprefs") == 0)
	{
		g_hPointCookie = RegClientCookie("MPoints", "Points used for ingame store", CookieAccess_Private);
		
		// late loading support for cookies
		for (int i=1; i<=MaxClients; i++)
		{
			if (AreClientCookiesCached(i))
			{
				OnClientCookiesCached(i);
			}
		}
	}
}

public void OnLibraryRemoved(const char[] name)
{
	if(strcmp(name, "clientprefs") == 0)
	{
		g_hCookieHidePanel = null;
	}
}

void GetMapSpecificPoints()
{
	 char mapName[256];
	 GetCurrentMap(mapName, sizeof(mapName));
	 for(new i=0; i<sizeof(mapName); ++i)
	 {
		  if(mapName[i] == '\0')
				break;
				
		  mapName[i] = CharToLower(mapName[i]);
	 }
	 
	char path[PLATFORM_MAX_PATH];
	BuildPath(Path_SM, path, sizeof(path),"configs/monster/maps/%s.txt", mapName);
	
	 int defaultPoints = g_hDefPoints.IntValue;
	 File file = OpenFile(path, "rt");
	 if (file == null)
	 {
		  return defaultPoints;
	 }
	 
	 while (file.ReadLine(buffer, sizeof(buffer)))
	 {
		  TrimString(buffer);
		  if (buffer[0] == '#')
				continue;
		  
		  defaultPoints = StringToInt(buffer);
		  break;
	 }
	 
	 file.Close();
	 return defaultPoints;
}

public int Native_GetPoints(Handle plugin, int numParams)
{
	int client = GetNativeCell(1);
	return g_iPoints[client];
}

public int Native_SetPoints(Handle plugin, int numParams)
{
	char buffer[8];
	 IntToString(points, buffer, sizeof(buffer));
	 g_iPoints[client] = points;
	 SetClientCookie(client, g_hPointCookie, buffer);
	 PrintToConsole(client, "[MONSTER] %s%d points (%s)", (points >= 0 ? "+" : "-"), points, reason);
	return 0;
}

bool CheckPointAwardConditions(int client)
{
	int team = GetClientTeam(client);
	if (team == 2 || team == 3) // 0 = unassigned, 1 = spectator, 2 = ct, 3 = t
	{
		return true;
	}
	return false;
}

public Action Timer_DeleteEntitySafe(Handle argTimer, any entRef)
{
	 int entity = EntRefToEntIndex(entRef);
	 if (entity != INVALID_ENT_REFERENCE)
	 {
		  AcceptEntityInput(entity, "Kill");
	 }
	return Plugin_Handled;
}

void AchievementEffect(int client)
{
	 if(IsPlayerAlive(client))
	 {
		  char parentName[32], targetName[32];	 
		  int particle = CreateEntityByName("info_particle_system");
		  if (particle > 0)
		{
				float pos[3];
				GetEntPropVector(client, Prop_Send, "m_vecOrigin", pos);
				TeleportEntity(particle, pos, NULL_VECTOR, NULL_VECTOR);

				FormatEx(parentName, sizeof(parentName), "target%i", client);
			FormatEx(targetName, sizeof(targetName), "monster_achievement%i", client);
				DispatchKeyValue(client, "targetname", parentName);
			DispatchKeyValue(particle, "targetname", targetName);

				DispatchKeyValue(particle, "parentname", parentName);
				DispatchKeyValue(particle, "effect_name", ACHIEVEMENT_PARTICLE);
				DispatchSpawn(particle);

				SetVariantString(name);
				AcceptEntityInput(particle, "SetParent", particle, particle, 0);
				SetVariantString("head");
				AcceptEntityInput(particle, "SetParentAttachment", particle, particle, 0);
				ActivateEntity(particle);
				AcceptEntityInput(particle, "start");
				
				CreateTimer(3.0, Timer_DeleteEntitySafe, EntIndexToEntRef(particle), TIMER_FLAG_NO_MAPCHANGE);
		  }
	 }
}

public Action Command_MapEnd(int args)
{
	 int points = GetMapSpecificPoints();
	 PrintToServer("[MONSTER] Map completed! Points: %d", points);
	 PrintToChatAll("\x04[Server] \x01Finish!");
	 PrintToChatAll("\x04[Server] \x01Thank you for playing this map");
	 PrintToChatAll("\x04[Server] \x01Awarding every player in a team \x03%d \x01points", points);
	for (int i=1; i<=MaxClients; i++)
	 {
		  if (IsValidPlayer(i) && CheckPointAwardConditions(i))
		  {
				Monster_SetPoints(i, points, "map completed");
				AchievementEffect(i);
		  }
	 }
	 return Plugin_Handled;
}

public Action:Command_Points(client, args)
{
	 if(client > 0)
	 {
		  PrintToChatAll("\x04[Server] \x01You currently have \x03%d \x01points.", Monster_GetPoints(client));
	 } else {
		  ReplyToCommand(client, "sm_points: cannot issue this cmd from server");
	 }
	 return Plugin_Handled;
}

bool IsValidPlayer(int client)
{
	 if (client>0 && client<=MaxClients && IsClientConnected(client) && IsClientInGame(client))
	 {
		  return true;
	 }
	 return false;
}